/**********************************************************************
 * FILE: mpi_pi_reduce.c
 * OTHER FILES: dboard.c
 * DESCRIPTION:
 *   pi Calculation Example - C Version
 *   This program calculates pi using a "dartboard" algorithm.  See
 *   Fox et al.(1988) Solving Problems on Concurrent Processors, vol.1
 *   page 207.  All processes contribute to the calculation, with the
 *   master averaging the values for pi. This version uses mpc_reduce to
 *   collect results
 * AUTHOR: Blaise Barney. Adapted from Ros Leibensperger, Cornell Theory
 *   Center. Converted to MPI: George L. Gusciora, MHPCC (1/95)
 * LAST REVISED: 06/13/13 Blaise Barney
 * version sans parallèlisme
 * Traduction en français des commentaires et modification pour travailler
 * à nombre total constant de lancers (IL 2017)
 **********************************************************************/
#include <omp.h>
#include <stdio.h>
#include <stdlib.h>

void srandom (unsigned seed);
double dboard (int darts,struct drand48_data *buf);
 
#define DARTS  2000000    /* Nombre de lancer de fléchettes dans la cible par tour  */
#define ROUNDS 1000      /* Nombre de tours de lancers */
#define MASTER 0       /* Numéro du « Maître » */



int main (int argc, char *argv[])
{
    double	homepi,         /* valeur de pi calculée par la tache courante */
            sumpi=0,
            pi,
            avepi;	       /* valeur moyenne de pi  */

    //int Nb_taches;
	//int seed;
    int Nb_taches=omp_get_num_threads();
    struct drand48_data drand_buf;
 
#pragma omp parallel private(drand_buf)
    {
    // zone paralelle
    /* la graine du générateur pseudo-aléatoire est fixée à la valeur du numéro de tache */
    //int taskid;
    int taskid = omp_get_thread_num();
    double r;
    //srandom (taskid);
    //srand48(taskid);
    int seed =1202107158+ taskid *1999;
    srand48_r(seed,&drand_buf);
#pragma omp barrier
    drand48_r(&drand_buf,&r);
    printf(" alea %f tache %i \n",r,taskid);
    avepi = 0;
#pragma omp for private(homepi)
    for (int i = 0; i < ROUNDS; i++)
        {
        if(sumpi!=0) printf("la tache %i a un sumpi diff de 0\n",taskid);
        homepi = dboard(DARTS,&drand_buf);
           
        sumpi +=homepi;
            
        if (taskid==MASTER)
        {
        avepi = ((avepi * i) + sumpi/Nb_taches)/(i + 1);
            //printf("%8d %10.8f %10.8f \n",(DARTS * (Nb_taches*(i + 1))),avepi,sumpi/Nb_taches);
            printf("%8d %10.8f %10.8f \n",(DARTS * (Nb_taches*(i + 1))),avepi,sumpi/Nb_taches);
            
        }
        sumpi=0;
    
        }
    }
    // partie monopro
    printf ("\nReal value of PI: 3.1415926535897 \n");
    
    return 0;
}

/**************************************************************************
 * subroutine dboard
 * DESCRIPTION:
 *   Used in pi calculation example codes.
 *   Throw darts at board.  Done by generating random numbers
 *   between 0 and 1 and converting them to values for x and y
 *   coordinates and then testing to see if they "land" in
 *   the circle."  If so, score is incremented.  After throwing the
 *   specified number of darts, pi is calculated.  The computed value
 *   of pi is returned as the value of this function, dboard.
 *
 *   Explanation of constants and variables used in this function:
 *   darts       = number of throws at dartboard
 *   score       = number of darts that hit circle
 *   n           = index variable
 *   r           = random number scaled between 0 and 1
 *   x_coord     = x coordinate, between -1 and 1
 *   y_coord     = y coordinate, between -1 and 1
 *   pi          = computed value of pi
 ****************************************************************************/

double dboard(int darts,struct drand48_data *buf)
{
#define sqr(x)	((x)*(x))
//        long random();
    
        double x_coord, y_coord, pi, r;
        int score, n;
  	// unsigned int cconst;  /* must be 4-bytes in size */
        /* 2 bit shifted to MAX_RAND later used to scale random number between 0 and 1 */
        //cconst = 2 << (31 - 1);
        score = 0;
        
        /* "throw darts at board" */
        for (n = 1; n <= darts; n++)  {
            /* generate random numbers for x and y coordinates */
            //r = (double)random()/cconst;
            //r=drand48();
            drand48_r(buf ,&r);
            x_coord = (2.0 * r) - 1.0;
            //r = (double)random()/cconst;
             // r=drand48();
	    drand48_r(buf ,&r);
            y_coord = (2.0 * r) - 1.0;
            
            /* if dart lands in circle, increment score */
            if ((sqr(x_coord) + sqr(y_coord)) <= 1.0)
                score++;
        }
    
    /* calculate pi */
    pi = 4.0 * (double)score/(double)darts;
    return(pi);
}
